@@ -25,7 +25,6 @@ entityref = re.compile('&([a-zA-Z][-.a-zA-Z0-9]*)[^a-zA-Z0-9]')
 charref = re.compile('&#(?:[0-9]+|[xX][0-9a-fA-F]+)[^0-9a-fA-F]')
 
 starttagopen = re.compile('<[a-zA-Z]')
-endtagopen = re.compile('</[a-zA-Z]')
 piclose = re.compile('>')
 commentclose = re.compile(r'--\s*>')
 # Note:
@@ -90,6 +89,7 @@ class HTMLParser(_markupbase.ParserBase):
         If convert_charrefs is True (the default), all character references
         are automatically converted to the corresponding Unicode characters.
         """
+        super().__init__()
         self.convert_charrefs = convert_charrefs
         self.reset()
 
@@ -99,7 +99,7 @@ class HTMLParser(_markupbase.ParserBase):
         self.lasttag = '???'
         self.interesting = interesting_normal
         self.cdata_elem = None
-        _markupbase.ParserBase.reset(self)
+        super().reset()
 
     def feed(self, data):
         r"""Feed data to the parser.
@@ -177,7 +177,7 @@ class HTMLParser(_markupbase.ParserBase):
                     k = self.parse_pi(i)
                 elif startswith("<!", i):
                     k = self.parse_html_declaration(i)
-                elif (i + 1) < n or end:
+                elif (i + 1) < n:
                     self.handle_data("<")
                     k = i + 1
                 else:
@@ -185,35 +185,17 @@ class HTMLParser(_markupbase.ParserBase):
                 if k < 0:
                     if not end:
                         break
-                    if starttagopen.match(rawdata, i):  # < + letter
-                        pass
-                    elif startswith("</", i):
-                        if i + 2 == n:
-                            self.handle_data("</")
-                        elif endtagopen.match(rawdata, i):  # </ + letter
-                            pass
-                        else:
-                            # bogus comment
-                            self.handle_comment(rawdata[i+2:])
-                    elif startswith("<!--", i):
-                        j = n
-                        for suffix in ("--!", "--", "-"):
-                            if rawdata.endswith(suffix, i+4):
-                                j -= len(suffix)
-                                break
-                        self.handle_comment(rawdata[i+4:j])
-                    elif startswith("<![CDATA[", i):
-                        self.unknown_decl(rawdata[i+3:])
-                    elif rawdata[i:i+9].lower() == '<!doctype':
-                        self.handle_decl(rawdata[i+2:])
-                    elif startswith("<!", i):
-                        # bogus comment
-                        self.handle_comment(rawdata[i+2:])
-                    elif startswith("<?", i):
-                        self.handle_pi(rawdata[i+2:])
+                    k = rawdata.find('>', i + 1)
+                    if k < 0:
+                        k = rawdata.find('<', i + 1)
+                        if k < 0:
+                            k = i + 1
                     else:
-                        raise AssertionError("we should not get here!")
-                    k = n
+                        k += 1
+                    if self.convert_charrefs and not self.cdata_elem:
+                        self.handle_data(unescape(rawdata[i:k]))
+                    else:
+                        self.handle_data(rawdata[i:k])
                 i = self.updatepos(i, k)
             elif startswith("&#", i):
                 match = charref.match(rawdata, i)
@@ -347,13 +329,6 @@ class HTMLParser(_markupbase.ParserBase):
 
         end = rawdata[k:endpos].strip()
         if end not in (">", "/>"):
-            lineno, offset = self.getpos()
-            if "\n" in self.__starttag_text:
-                lineno = lineno + self.__starttag_text.count("\n")
-                offset = len(self.__starttag_text) \
-                         - self.__starttag_text.rfind("\n")
-            else:
-                offset = offset + len(self.__starttag_text)
             self.handle_data(rawdata[i:endpos])
             return endpos
         if end.endswith('/>'):
