@@ -87,7 +87,18 @@ On Debian and Ubuntu systems, cc_apt_configure will write a deb822 compatible
 to write /etc/apt/sources.list directly.
 """
 
-DEPRECATION_INFO_BOUNDARY = "24.1"
+MANUAL_NETWORK_WAIT = False
+"""
+On Ubuntu systems, cloud-init-network.service will start immediately after
+cloud-init-local.service and manually wait for network online when necessary.
+If False, rely on systemd ordering to ensure network is available before
+starting cloud-init-network.service.
+
+Note that in addition to this flag, downstream patches are also likely needed
+to modify the systemd unit files.
+"""
+
+DEPRECATION_INFO_BOUNDARY = "24.3"
 """
 DEPRECATION_INFO_BOUNDARY is used by distros to configure at which upstream
 version to start logging deprecations at a level higher than INFO.
@@ -107,6 +118,11 @@ that newly added deprecations will be logged at INFO level. The implication of
 the different log levels is that logs at DEPRECATED level result in a return
 code of 2 from `cloud-init status`.
 
+This may may also be used in some limited cases where new error messages may be
+logged which increase the risk of regression in stable downstreams where the
+error was previously unreported yet downstream users expected stable behavior
+across new cloud-init releases.
+
 format:
 
 <value> :: = <default> | <version>
